@@ -2,13 +2,14 @@
 #' 
 #' Draw samples from the posterior predictive distribution for future 
 #' time points given the posterior draws of hyperparameters \eqn{\theta} and 
-#' latent state \eqn{alpha_{n+1}}. Function can also be used to draw samples 
-#' from the posterior predictive distribution
-#'  \eqn{p(\tilde y_1, \ldots, \tilde y_n | y_1,\ldots, y_n)}.
-#'
+#' latent state \eqn{alpha_{n+1}} returned by \code{run_mcmc}. 
+#' Function can also be used to draw samples from the posterior predictive 
+#' distribution \eqn{p(\tilde y_1, \ldots, \tilde y_n | y_1,\ldots, y_n)}.
+#' 
+#' @seealso \code{fitted} for in-sample predictions.
 #' @param object Results object of class \code{mcmc_output} from 
-#' \code{\link{run_mcmc}}
-#' @param model A \code{bssm_model} object.. 
+#' \code{\link{run_mcmc}}.
+#' @param model A \code{bssm_model} object.
 #' Should have same structure and class as the original model which was used in 
 #' \code{run_mcmc}, in order to plug the posterior samples of the model 
 #' parameters to the right places. 
@@ -17,16 +18,19 @@
 #' \code{future} to \code{FALSE}.
 #' @param type Type of predictions. Possible choices are 
 #' \code{"mean"} \code{"response"}, or  \code{"state"} level. 
-#' @param nsim Positive integer defining number of samples to draw.
+#' @param nsim Positive integer defining number of samples to draw. Should be 
+#' less than or equal to \code{sum(object$counts)} i.e. the number of samples 
+#' in the MCMC output. Default is to use all the samples.
 #' @param future Default is \code{TRUE}, in which case predictions are for the 
 #' future, using posterior samples of (theta, alpha_T+1) i.e. the 
 #' posterior samples of hyperparameters and latest states. 
 #' Otherwise it is assumed that \code{model} corresponds to the original model.
-#' @param seed Seed for RNG (positive integer). Note that this affects only the 
-#' C++ side, and \code{predict} also uses R side RNG for subsampling, so for 
-#' replicable results you should call \code{set.seed} before \code{predict}.
+#' @param seed Seed for the C++ RNG (positive integer). Note that this affects 
+#' only the C++ side, and \code{predict} also uses R side RNG for subsampling, 
+#' so for replicable results you should call \code{set.seed} before 
+#' \code{predict}.
 #' @param ... Ignored.
-#' @return A \code{data.frame} consisting of samples from the predictive 
+#' @return A data.frame consisting of samples from the predictive 
 #' posterior distribution.
 #' @method predict mcmc_output
 #' @aliases predict predict.mcmc_output
@@ -44,7 +48,7 @@
 #'   start = tsp(model$y)[2] + 2 * deltat(model$y), 
 #'   frequency = frequency(model$y))
 #' # use "state" for illustrative purposes, we could use type = "mean" directly
-#' pred <- predict(mcmc_results, future_model, type = "state", 
+#' pred <- predict(mcmc_results, model = future_model, type = "state", 
 #'   nsim = 1000)
 #' 
 #' library("dplyr")
@@ -87,9 +91,9 @@
 #'     time = time(JohnsonJohnson)))
 #' 
 #' # Posterior predictions for past observations:
-#' yrep <- predict(mcmc_results, model, type = "response", 
+#' yrep <- predict(mcmc_results, model = model, type = "response", 
 #'   future = FALSE, nsim = 1000)
-#' meanrep <- predict(mcmc_results, model, type = "mean", 
+#' meanrep <- predict(mcmc_results, model = model, type = "mean", 
 #'   future = FALSE, nsim = 1000)
 #'   
 #' sumr_yrep <- yrep %>% 
@@ -107,7 +111,8 @@
 #'   mutate(interval = "Mean")
 #'     
 #' rbind(sumr_meanrep, sumr_yrep) %>% 
-#'   mutate(interval = factor(interval, levels = c("Observations", "Mean"))) %>%
+#'   mutate(interval = 
+#'     factor(interval, levels = c("Observations", "Mean"))) %>%
 #'   ggplot(aes(x = time, y = earnings)) + 
 #'   geom_ribbon(aes(ymin = lwr, ymax = upr, fill = interval), 
 #'    alpha = 0.75) +
@@ -120,16 +125,11 @@
 predict.mcmc_output <- function(object, model, nsim, type = "response",  
   future = TRUE, seed = sample(.Machine$integer.max, size = 1), ...) {
   
+  check_missingness(model)
+  
   if (!inherits(model, "bssm_model")) {
-    stop("Argument 'model' should be of class 'bssm_model'. ")
+    stop("Argument 'model' should be an object of class 'bssm_model'.")
   }
-  nsim <- check_integer(nsim, "nsim")
-  seed <- check_integer(seed, "seed", FALSE, max = .Machine$integer.max)
-  
-  if (!test_flag(future)) stop("Argument 'future' should be TRUE or FALSE. ")
-  
-  type <- match.arg(type, c("response", "mean", "state"))
-  
   if (object$output_type != 1) 
     stop("MCMC output must contain posterior samples of the states.")
   
@@ -140,7 +140,19 @@ predict.mcmc_output <- function(object, model, nsim, type = "response",
     stop(paste("Number of unknown parameters 'theta' does not correspond to",
       "the MCMC output. ", sep = " "))
   }
-  if (nsim < 1) stop("Number of samples 'nsim' should be at least one.")
+  if (missing(nsim)) {
+    nsim <- sum(object$counts)
+  } else {
+    nsim <- check_intmax(nsim, "nsim", positive = TRUE, max = Inf)
+    if (nsim > sum(object$count))
+      stop(paste0("The number of samples should be smaller than or equal to ",
+        "the number of posterior samples ", sum(object$counts), "."))
+  }
+  seed <- check_intmax(seed, "seed", FALSE, max = .Machine$integer.max)
+  
+  if (!test_flag(future)) stop("Argument 'future' should be TRUE or FALSE. ")
+  
+  type <- match.arg(tolower(type), c("response", "mean", "state"))
   
   if (attr(object, "model_type") %in% c("bsm_lg", "bsm_ng")) {
     object$theta[, 1:(ncol(object$theta) - length(model$beta))] <- 
@@ -160,15 +172,19 @@ predict.mcmc_output <- function(object, model, nsim, type = "response",
     }
   }
   
+  idx <- sample(seq_len(sum(object$counts)), size = nsim)
+  if (object$mcmc_type %in% paste0("is", 1:3)) {
+    weight <- rep(object$weights, times = object$counts)[idx]
+  } else {
+    weight <- rep(1, nsim)
+  }
+  theta <- 
+    t(apply(object$theta, 2, rep, times = object$counts)[idx, , drop = FALSE])
+  
   if (future) {
     
-    w <- object$counts * 
-      (if (object$mcmc_type %in% paste0("is", 1:3)) object$weights else 1)
-    idx <- sample(seq_len(nrow(object$theta)), size = nsim, prob = w, 
-      replace = TRUE)
-    theta <- t(object$theta[idx, , drop = FALSE])
-    alpha <- matrix(object$alpha[nrow(object$alpha), , idx], 
-      nrow = ncol(object$alpha))
+    states <- t(apply(object$alpha[nrow(object$alpha), , , drop = FALSE], 
+      2, rep, object$counts)[idx, , drop = FALSE])
     
     switch(attr(object, "model_type"),
       ssm_mlg =,
@@ -179,7 +195,7 @@ predict.mcmc_output <- function(object, model, nsim, type = "response",
           stop(paste("Model does not correspond to the MCMC output:",
             "Wrong number of states. ", sep = " "))
         }
-        pred <- gaussian_predict(model, theta, alpha,
+        pred <- gaussian_predict(model, theta, states,
           pmatch(type, c("response", "mean", "state")), 
           seed, 
           pmatch(attr(object, "model_type"), 
@@ -199,7 +215,7 @@ predict.mcmc_output <- function(object, model, nsim, type = "response",
           c("svm", "poisson", "binomial", "negative binomial", "gamma", 
             "gaussian"), 
           duplicates.ok = TRUE) - 1
-        pred <- nongaussian_predict(model, theta, alpha,
+        pred <- nongaussian_predict(model, theta, states,
           pmatch(type, c("response", "mean", "state")), seed, 
           pmatch(attr(object, "model_type"), 
             c("ssm_mng", "ssm_ung", "bsm_ng", "svm", "ar1_ng")) - 1L)
@@ -218,10 +234,11 @@ predict.mcmc_output <- function(object, model, nsim, type = "response",
           model$log_prior_pdf, model$known_params, 
           model$known_tv_params, as.integer(model$time_varying),
           model$n_states, model$n_etas,
-          theta, alpha, pmatch(type, c("response", "mean", "state")), seed)
+          theta, states, pmatch(type, c("response", "mean", "state")), seed)
         
       }
       , stop("Not yet implemented for ssm_sde. "))
+    
     if (type == "state") {
       if (attr(object, "model_type") == "ssm_nlg") {
         variables <- model$state_names
@@ -236,27 +253,26 @@ predict.mcmc_output <- function(object, model, nsim, type = "response",
     d <- data.frame(value = as.numeric(pred),
       variable = variables,
       time = rep(time(model$y), each = nrow(pred)),
+      weight = rep(weight, each = nrow(pred) * ncol(pred)),
       sample = rep(1:nsim, each = nrow(pred) * ncol(pred)))
     
   } else {
     
     if (inherits(model, c("ssm_mng", "ssm_mlg", "ssm_nlg"))) {
       if (!identical(nrow(object$alpha) - 1L, nrow(model$y))) {
-        stop("Number of observations of the model and MCMC output do not match.") 
+        stop(paste0("Number of observations in the model and MCMC output do ", 
+          "not match."))
       }
     } else {
       if (!identical(nrow(object$alpha) - 1L, length(model$y))) {
-        stop("Number of observations of the model and MCMC output do not match.") 
+        stop(paste0("Number of observations in the model and MCMC output do ", 
+          "not match."))
       }
     }
-    w <- object$counts * 
-      (if (object$mcmc_type %in% paste0("is", 1:3)) object$weights else 1)
-    idx <- sample(seq_len(nrow(object$theta)), size = nsim, prob = w, 
-      replace = TRUE)
-    n <- nrow(object$alpha) - 1L
-    m <- ncol(object$alpha)
     
-    states <- object$alpha[1:n, , idx, drop = FALSE]
+    n <- nrow(object$alpha) - 1L
+    states <- 
+      apply(object$alpha, 1:2, rep, object$counts)[idx, 1:n, , drop = FALSE]
     
     if (type == "state") {
       if (attr(object, "model_type") == "ssm_nlg") {
@@ -265,24 +281,23 @@ predict.mcmc_output <- function(object, model, nsim, type = "response",
         variables <- names(model$a1)
       }
       d <- data.frame(value = as.numeric(states),
-        variable = rep(variables, each = n),
-        time = rep(time(model$y), times = m),
-        sample = rep(1:nsim, each = n * m))
+        variable = rep(variables, each = nrow(states) * n),
+        time = rep(time(model$y), times = nrow(states)),
+        sample = 1:nsim, weight = weight)
     } else {
       
       variables <- colnames(model$y)
       if (is.null(variables)) 
         variables <- paste("Series", 1:max(1, ncol(model$y)))
-      
-      theta <- t(object$theta[idx, ])
-      states <- aperm(states, c(2, 1, 3))
+     
+      states <- aperm(states, 3:1)
       
       switch(attr(object, "model_type"),
         ssm_mlg =,
         ssm_ulg =,
         bsm_lg =,
         ar1_lg = {
-          if (!identical(length(model$a1), m)) {
+          if (!identical(length(model$a1), nrow(states))) {
             stop(paste("Model does not correspond to the MCMC output:",
               "Wrong number of states. ", sep = " "))
           }
@@ -298,7 +313,7 @@ predict.mcmc_output <- function(object, model, nsim, type = "response",
         bsm_ng =, 
         svm =,
         ar1_ng = {
-          if (!identical(length(model$a1), m)) {
+          if (!identical(length(model$a1), nrow(states))) {
             stop(paste("Model does not correspond to the MCMC output:",
               "Wrong number of states. ", sep = " "))
           }
@@ -315,7 +330,7 @@ predict.mcmc_output <- function(object, model, nsim, type = "response",
             warning("NA or NaN values in predictions, possible under/overflow?")
         },
         ssm_nlg = {
-          if (!identical(model$n_states, m)) {
+          if (!identical(model$n_states, nrow(states))) {
             stop(paste("Model does not correspond to the MCMC output:",
               "Wrong number of states. ", sep = " "))
           }
@@ -333,7 +348,8 @@ predict.mcmc_output <- function(object, model, nsim, type = "response",
       d <- data.frame(value = as.numeric(pred),
         variable = variables,
         time = rep(time(model$y), each = nrow(pred)),
-        sample = rep(1:nsim, each = nrow(pred) * ncol(pred)))
+        sample = rep(1:nsim, each = nrow(pred) * ncol(pred)),
+        weight = rep(weight, each = nrow(pred) * ncol(pred)))
     }
   }
   d
