@@ -17,10 +17,11 @@ class approx_mcmc: public mcmc {
 public:
 
   // constructor
-  approx_mcmc(const unsigned int iter, const unsigned int burnin, const unsigned int thin,
-    const unsigned int n, const unsigned int m, const unsigned int p, 
-    const double target_acceptance, const double gamma, const arma::mat& S, 
-    const unsigned int output_type = 1, const bool store_modes = true);
+  approx_mcmc(const unsigned int iter, const unsigned int burnin, 
+    const unsigned int thin, const unsigned int n, const unsigned int m, 
+    const unsigned int p, const double target_acceptance, const double gamma, 
+    const arma::mat& S, const unsigned int output_type = 1, 
+    const bool store_modes = true, const bool verbose = true);
 
   void expand();
 
